from dataclasses import dataclass
from typing import TYPE_CHECKING, Callable, List, Optional, Set, TypeVar, cast

from arcor2 import rest
from arcor2.data.common import ActionMetadata, Joint, Pose, ProjectRobotJoints, StrEnum
from arcor2.object_types.abstract import Robot
from arcor2.object_types.abstract import Settings as BaseSettings


@dataclass
class Settings(BaseSettings):
    url: str
    configuration_id: str


# mypy work-around by GvR (https://github.com/python/mypy/issues/5107#issuecomment-529372406)
if TYPE_CHECKING:
    F = TypeVar("F", bound=Callable)

    def lru_cache(maxsize: int = 128, typed: bool = False) -> Callable[[F], F]:
        pass


else:
    from functools import lru_cache


class MoveTypeEnum(StrEnum):

    LINE: str = "Line"
    SIMPLE: str = "Simple"


class AbstractRobot(Robot):
    """Object (with pose) that serves as a base for Robot service.

    There are methods/action that are common to robots with (Aubo) and
    without (Simatic) end effectors.
    """

    def __init__(self, obj_id: str, name: str, pose: Pose, settings: Settings) -> None:
        super(AbstractRobot, self).__init__(obj_id, name, pose, settings)
        rest.call(
            rest.Method.PUT,
            f"{self.settings.url}/system/set",
            body=pose,
            params={"configId": self.settings.configuration_id, "id": self.id},
        )

        self.move_timeout = rest.Timeout(read=5 * 60)

    @property
    def settings(self) -> Settings:
        return cast(Settings, super(AbstractRobot, self).settings)

    def configurations(self) -> Set[str]:
        return set(rest.call(rest.Method.GET, f"{self.settings.url}/configurations", list_return_type=str))

    def cleanup(self) -> None:
        super(AbstractRobot, self).cleanup()
        rest.call(rest.Method.PUT, f"{self.settings.url}/system/reset")

    # --- Grippers Controller ------------------------------------------------------------------------------------------

    @lru_cache()
    def grippers(self) -> Set[str]:
        return set(rest.call(rest.Method.GET, f"{self.settings.url}/grippers", list_return_type=str))

    def grip(
        self,
        gripper_id: str,
        position: float = 0.0,
        speed: float = 0.5,
        force: float = 0.5,
        *,
        an: Optional[str] = None,
    ) -> None:

        assert 0.0 <= position <= 1.0
        assert 0.0 <= speed <= 1.0
        assert 0.0 <= force <= 1.0

        rest.call(
            rest.Method.PUT,
            f"{self.settings.url}/grippers/{gripper_id}/grip",
            params={"position": position, "speed": speed, "force": force},
        )

    def set_opening(
        self, gripper_id: str, position: float = 1.0, speed: float = 0.5, *, an: Optional[str] = None
    ) -> None:

        assert 0.0 <= position <= 1.0
        assert 0.0 <= speed <= 1.0

        rest.call(
            rest.Method.PUT,
            f"{self.settings.url}/grippers/{gripper_id}/opening",
            params={"position": position, "speed": speed},
        )

    def get_gripper_opening(self, gripper_id: str, *, an: Optional[str] = None) -> float:

        return rest.call(rest.Method.GET, f"{self.settings.url}/grippers/{gripper_id}/opening", return_type=float)

    def is_item_gripped(self, gripper_id: str, *, an: Optional[str] = None) -> bool:
        return rest.call(rest.Method.GET, f"{self.settings.url}/grippers/{gripper_id}/gripped", return_type=bool)

    # --- IOs Controller -----------------------------------------------------------------------------------------------

    @lru_cache()
    def inputs(self) -> Set[str]:
        return set(rest.call(rest.Method.GET, f"{self.settings.url}/inputs", return_type=str))

    @lru_cache()
    def outputs(self) -> Set[str]:
        return set(rest.call(rest.Method.GET, f"{self.settings.url}/outputs", return_type=str))

    def get_input(self, input_id: str, *, an: Optional[str] = None) -> float:
        return rest.call(rest.Method.GET, f"{self.settings.url}/inputs/{input_id}", return_type=float)

    def set_output(self, output_id: str, value: float, *, an: Optional[str] = None) -> None:

        assert 0.0 <= value <= 1.0
        rest.call(rest.Method.PUT, f"{self.settings.url}/outputs/{output_id}", params={"value": value})

    def get_output(self, output_id: str, *, an: Optional[str] = None) -> float:
        return rest.call(rest.Method.GET, f"{self.settings.url}/outputs/{output_id}", return_type=float)

    # --- Joints Controller --------------------------------------------------------------------------------------------

    def set_joints(
        self,
        joints: ProjectRobotJoints,
        move_type: MoveTypeEnum,
        speed: float = 0.5,
        acceleration: float = 0.5,
        safe: bool = True,
        *,
        an: Optional[str] = None,
    ) -> None:

        assert 0.0 <= speed <= 1.0
        assert 0.0 <= acceleration <= 1.0

        url = f"{self.settings.url}/joints"
        params = {"moveType": move_type.value, "speed": speed, "acceleration": acceleration, "safe": safe}

        with self._move_lock:
            rest.call(rest.Method.PUT, url, body=joints.joints, params=params, timeout=self.move_timeout)

    def robot_joints(self) -> List[Joint]:
        return rest.call(rest.Method.GET, f"{self.settings.url}/joints", list_return_type=Joint)

    # --- Robot Controller ---------------------------------------------------------------------------------------------

    @lru_cache()
    def moves(self) -> Set[str]:
        return set(rest.call(rest.Method.GET, f"{self.settings.url}/moves", list_return_type=str))

    def stop(self) -> None:
        rest.call(rest.Method.PUT, f"{self.settings.url}/stop")

    set_joints.__action__ = ActionMetadata(blocking=True)  # type: ignore
    get_input.__action__ = ActionMetadata(blocking=True)  # type: ignore
    set_output.__action__ = ActionMetadata(blocking=True)  # type: ignore
    get_output.__action__ = ActionMetadata(blocking=True)  # type: ignore
    grip.__action__ = ActionMetadata(blocking=True)  # type: ignore
    set_opening.__action__ = ActionMetadata(blocking=True)  # type: ignore
    get_gripper_opening.__action__ = ActionMetadata(blocking=True)  # type: ignore
    is_item_gripped.__action__ = ActionMetadata(blocking=True)  # type: ignore


AbstractRobot.CANCEL_MAPPING = {
    AbstractRobot.set_joints.__name__: AbstractRobot.stop.__name__,
}


__all__ = [Settings.__name__, AbstractRobot.__name__]



from dataclasses import dataclass
from typing import Optional, cast

from arcor2 import rest
from arcor2.data.common import Pose
from arcor2.data.object_type import Models
from arcor2.object_types.abstract import GenericWithPose
from arcor2.object_types.abstract import Settings as BaseSettings


@dataclass
class Settings(BaseSettings):
    url: str
    configuration_id: str


class AbstractWithPose(GenericWithPose):
    """Object (without pose) that serves as a base class for feature services
    with 'Configurations' and 'System' controller."""

    def __init__(
        self,
        obj_id: str,
        name: str,
        pose: Pose,
        collision_model: Optional[Models] = None,
        settings: Optional[Settings] = None,
    ) -> None:
        super(AbstractWithPose, self).__init__(obj_id, name, pose, collision_model, settings)
        rest.call(
            rest.Method.PUT,
            f"{self.settings.url}/system/set",
            body=pose,
            params={"configId": self.settings.configuration_id, "id": self.id},
        )

    @property
    def settings(self) -> Settings:
        return cast(Settings, super(AbstractWithPose, self).settings)

    def cleanup(self) -> None:
        super(AbstractWithPose, self).cleanup()
        rest.call(rest.Method.PUT, f"{self.settings.url}/system/reset")


__all__ = [
    Settings.__name__,
    AbstractWithPose.__name__,
]



from dataclasses import dataclass
from typing import List, Optional

from dataclasses_jsonschema import JsonSchemaMixin

from arcor2 import rest
from arcor2.data.common import ActionMetadata, StrEnum

from .abstract_simple import AbstractSimple


class NotificationLevelEnum(StrEnum):

    INFO: str = "Info"
    WARN: str = "Warn"
    ERROR: str = "Error"


@dataclass
class DialogValue(JsonSchemaMixin):

    title: str
    content: str
    options: List[str]


@dataclass
class NotificationValue(JsonSchemaMixin):

    message: str
    level: NotificationLevelEnum
    created: int


class Interaction(AbstractSimple):
    """REST interface to the Interaction service (0.1.1)."""

    _ABSTRACT = False

    # --- Dialog Controller --------------------------------------------------------------------------------------------

    def add_dialog(self, title: str, content: str, options: List[str], *, an: Optional[str] = None) -> str:
        """Creates dialog, block until operator selects one of the options.

        :param title: Dialog title
        :param content: Dialog content
        :param options: Dialog options.
        :return:
        """

        return rest.call(
            rest.Method.PUT,
            f"{self.settings.url}/dialog",
            return_type=str,
            params={"title": title, "content": content},
            body=options,
            timeout=rest.Timeout(read=8 * 60 * 60),
        )

    def get_dialog(self, *, an: Optional[str] = None) -> DialogValue:
        """Returns active dialog (title and options) or error if no dialog is
        active.

        :return:
        """

        return rest.call(rest.Method.GET, f"{self.settings.url}/dialog", return_type=DialogValue)

    def resolve_dialog(self, option: str, *, an: Optional[str] = None) -> None:
        """Resolves current dialog using one of the options.

        :param option:
        :return:
        """

        rest.call(rest.Method.PUT, f"{self.settings.url}/dialog/resolve", params={"option": option})

    # --- Notification Controller --------------------------------------------------------------------------------------

    def add_notification(self, message: str, level: NotificationLevelEnum, *, an: Optional[str] = None) -> None:
        """Creates notification.

        :param message:
        :param level:
        :return:
        """

        rest.call(rest.Method.PUT, f"{self.settings.url}/notification", params={"message": message, "level": level})

    def delete_notifications(self, *, an: Optional[str] = None) -> None:
        """Clears all notifications.

        :return:
        """

        rest.call(rest.Method.DELETE, f"{self.settings.url}/notifications")

    def get_notifications(self, since_timestamp: int, *, an: Optional[str] = None) -> List[NotificationValue]:
        """Gets all notifications stored after given timestamp.

        :param since_timestamp: UNIX timestamp in nanoseconds, after which created notifications are returned.
        :return:
        """
        return rest.call(
            rest.Method.GET,
            f"{self.settings.url}/notifications",
            list_return_type=NotificationValue,
            params={"sinceTimestamp": since_timestamp},
        )

    add_dialog.__action__ = ActionMetadata(blocking=True)  # type: ignore
    get_dialog.__action__ = ActionMetadata(blocking=True)  # type: ignore
    resolve_dialog.__action__ = ActionMetadata(blocking=True)  # type: ignore
    add_notification.__action__ = ActionMetadata(blocking=True)  # type: ignore
    delete_notifications.__action__ = ActionMetadata(blocking=True)  # type: ignore
    get_notifications.__action__ = ActionMetadata(blocking=True)  # type: ignore



from dataclasses import dataclass
from typing import List, Optional, Set

from dataclasses_jsonschema import JsonSchemaMixin

from arcor2 import DynamicParamTuple as DPT
from arcor2 import rest
from arcor2.data.common import ActionMetadata, Joint, Orientation, Pose, Position, ProjectRobotJoints, RelativePose

from .abstract_robot import AbstractRobot, MoveTypeEnum, lru_cache


@dataclass
class MoveRelativeParameters(JsonSchemaMixin):

    pose: Pose
    position: Position
    orientation: Orientation


@dataclass
class MoveRelativeJointsParameters(JsonSchemaMixin):

    joints: List[Joint]
    position: Position
    orientation: Orientation


class Aubo(AbstractRobot):
    """REST interface to the robot service (0.7.0)."""

    _ABSTRACT = False
    urdf_package_name = "aubo.zip"

    def move_to_pose(self, end_effector_id: str, target_pose: Pose, speed: float) -> None:
        self.move(end_effector_id, target_pose, MoveTypeEnum.SIMPLE, speed, safe=True)

    def move_to_joints(self, target_joints: List[Joint], speed: float) -> None:
        self.set_joints(ProjectRobotJoints("", "", "", target_joints), MoveTypeEnum.SIMPLE, speed, safe=True)

    # --- EndEffectors Controller --------------------------------------------------------------------------------------

    @lru_cache()
    def get_end_effectors_ids(self) -> Set[str]:
        return set(rest.call(rest.Method.GET, f"{self.settings.url}/endEffectors", list_return_type=str))

    def get_end_effector_pose(self, end_effector_id: str) -> Pose:
        return rest.call(rest.Method.GET, f"{self.settings.url}/endEffectors/{end_effector_id}/pose", return_type=Pose)

    def move(
        self,
        end_effector_id: str,
        pose: Pose,
        move_type: MoveTypeEnum,
        speed: float = 0.5,
        acceleration: float = 0.5,
        safe: bool = True,
        *,
        an: Optional[str] = None,
    ) -> None:
        """Moves the robot's end-effector to a specific pose.

        :param end_effector_id: Unique end-effector id.
        :param pose: Target pose.
        :param move_type: Type of move.
        :param speed: Speed of move.
        :param acceleration: Acceleration of move.
        :param safe: When true, robot will consider its environment and avoid collisions.
        :return:
        """

        assert 0.0 <= speed <= 1.0
        assert 0.0 <= acceleration <= 1.0

        url = f"{self.settings.url}/endEffectors/{end_effector_id}/move"

        params = {"moveType": move_type.value, "speed": speed, "acceleration": acceleration, "safe": safe}

        with self._move_lock:
            rest.call(rest.Method.PUT, url, body=pose, params=params, timeout=self.move_timeout)

    def move_relative(
        self,
        end_effector_id: str,
        pose: Pose,
        rel_pose: RelativePose,
        move_type: MoveTypeEnum,
        speed: float = 0.5,
        acceleration: float = 0.5,
        safe: bool = True,
        *,
        an: Optional[str] = None,
    ) -> None:
        """Moves the robot's end-effector to a specific pose.

        :param end_effector_id: Unique end-effector id.
        :param pose: Target pose.
        :param rel_pose: Relative pose.
        :param move_type: Type of move.
        :param speed: Speed of move.
        :param acceleration: Acceleration of move.
        :param safe: When true, robot will consider its environment and avoid collisions.
        :return:
        """

        assert 0.0 <= speed <= 1.0
        assert 0.0 <= acceleration <= 1.0

        url = f"{self.settings.url}/endEffectors/{end_effector_id}/moveRelative"

        params = {"moveType": move_type.value, "speed": speed, "acceleration": acceleration, "safe": safe}
        body = MoveRelativeParameters(pose, rel_pose.position, rel_pose.orientation)

        with self._move_lock:
            rest.call(rest.Method.PUT, url, body=body, params=params, timeout=self.move_timeout)

    def move_relative_joints(
        self,
        end_effector_id: str,
        joints: ProjectRobotJoints,
        rel_pose: RelativePose,
        move_type: MoveTypeEnum,
        speed: float = 0.5,
        acceleration: float = 0.5,
        safe: bool = True,
        *,
        an: Optional[str] = None,
    ) -> None:
        """Moves the robot's end-effector relatively to specific joint values.

        :param robot_id: Unique robot id.
        :param end_effector_id: Unique end-effector id.
        :param joints: Target joints.
        :param rel_pose: Relative pose.
        :param move_type: Type of move.
        :param speed: Speed of move.
        :param acceleration: Acceleration of move.
        :param safe: When true, robot will consider its environment and avoid collisions.
        :return:
        """

        assert 0.0 <= speed <= 1.0
        assert 0.0 <= acceleration <= 1.0

        url = f"{self.settings.url}/endEffectors/{end_effector_id}/moveJointsRelative"

        body = MoveRelativeJointsParameters(joints.joints, rel_pose.position, rel_pose.orientation)
        params = {"moveType": move_type.value, "speed": speed, "acceleration": acceleration, "safe": safe}

        with self._move_lock:
            rest.call(rest.Method.PUT, url, body=body, params=params, timeout=self.move_timeout)

    def link(self, end_effector_id: str, collision_id: str) -> None:
        """Links collision object to end effector.

        :param end_effector_id: Unique end-effector id.
        :param collision_id: Unique id of collision
        :return:
        """

        rest.call(
            rest.Method.PUT,
            f"{self.settings.url}/endEffectors/{end_effector_id}/link",
            params={"collisionId": collision_id},
        )

    def unlink(self, end_effector_id: str) -> None:
        """Unlinks collision object from end effector.

        :param end_effector_id: Unique end-effector id.
        :return:
        """

        rest.call(rest.Method.PUT, f"{self.settings.url}/endEffectors/{end_effector_id}/unlink")

    def inverse_kinematics(
        self,
        end_effector_id: str,
        pose: Pose,
        start_joints: Optional[List[Joint]] = None,
        avoid_collisions: bool = True,
    ) -> List[Joint]:
        """Computes inverse kinematics.

        :param end_effector_id: IK target pose end-effector
        :param pose: IK target pose
        :param start_joints: IK start joints (if not provided, current joint values will be used)
        :param avoid_collisions: Return non-collision IK result if true
        :return: Inverse kinematics
        """

        if start_joints is None:
            start_joints = self.robot_joints()

        return rest.call(
            rest.Method.PUT,
            f"{self.settings.url}/endEffectors/{end_effector_id}/inverseKinematics",
            params={"startJoints": [joint.to_dict() for joint in start_joints], "avoidCollisions": avoid_collisions},
            body=pose,
            list_return_type=Joint,
        )

    def forward_kinematics(self, end_effector_id: str, joints: List[Joint]) -> Pose:
        """Computes forward kinematics.

        :param end_effector_id: Target end effector name
        :param joints: Input joint values
        :return: Pose of the given end effector
        """
        return rest.call(
            rest.Method.PUT,
            f"{self.settings.url}/endEffectors/{end_effector_id}/forwardKinematics",
            body=joints,
            return_type=Pose,
        )

    # --- Suctions Controller ------------------------------------------------------------------------------------------

    @lru_cache()
    def suctions(self) -> Set[str]:
        return set(rest.call(rest.Method.GET, f"{self.settings.url}/suctions", return_type=str))

    def suck(self, suction_id: str, *, an: Optional[str] = None) -> None:
        rest.call(rest.Method.PUT, f"{self.settings.url}/suctions/{suction_id}/suck")

    def release(self, suction_id: str, *, an: Optional[str] = None) -> None:
        rest.call(rest.Method.PUT, f"{self.settings.url}/suctions/{suction_id}/release")

    def is_item_attached(self, suction_id: str, *, an: Optional[str] = None) -> bool:
        return rest.call(rest.Method.GET, f"{self.settings.url}/suctions/{suction_id}/attached", return_type=bool)

    move.__action__ = ActionMetadata(blocking=True)  # type: ignore
    move_relative.__action__ = ActionMetadata(blocking=True)  # type: ignore
    move_relative_joints.__action__ = ActionMetadata(blocking=True)  # type: ignore
    suck.__action__ = ActionMetadata(blocking=True)  # type: ignore
    release.__action__ = ActionMetadata(blocking=True)  # type: ignore
    is_item_attached.__action__ = ActionMetadata(blocking=True)  # type: ignore


Aubo.DYNAMIC_PARAMS = {
    "end_effector_id": DPT(Aubo.get_end_effectors_ids.__name__, set()),
    "gripper_id": DPT(Aubo.grippers.__name__, set()),
    "suction_id": DPT(Aubo.suctions.__name__, set()),
    "input_id": DPT(Aubo.inputs.__name__, set()),
    "output_id": DPT(Aubo.outputs.__name__, set()),
}

Aubo.CANCEL_MAPPING.update(
    {
        Aubo.move.__name__: Aubo.stop.__name__,
        Aubo.move_relative.__name__: Aubo.stop.__name__,
        Aubo.move_relative_joints.__name__: Aubo.stop.__name__,
    }
)



from dataclasses import dataclass
from typing import cast

from arcor2.object_types.abstract import Generic
from arcor2.object_types.abstract import Settings as BaseSettings


@dataclass
class Settings(BaseSettings):
    url: str


class AbstractSimple(Generic):
    """Object (without pose) that serves as a base class for feature services
    without 'Configurations' and 'System' controllers."""

    def __init__(self, obj_id: str, name: str, settings: Settings) -> None:
        super(AbstractSimple, self).__init__(obj_id, name, settings)

    @property
    def settings(self) -> Settings:
        return cast(Settings, super(AbstractSimple, self).settings)


__all__ = [
    Settings.__name__,
    AbstractSimple.__name__,
]



import os

from arcor2 import package_version

_ROOT = os.path.abspath(os.path.dirname(__file__))


def version() -> str:
    return package_version(__name__)


def get_data(path: str) -> str:
    return os.path.join(_ROOT, "data", path)



#!/usr/bin/env python3

from arcor2.object_types.upload import Urdf, upload_def
from arcor2_kinali import get_data
from arcor2_kinali.object_types.abstract_robot import AbstractRobot
from arcor2_kinali.object_types.abstract_simple import AbstractSimple
from arcor2_kinali.object_types.abstract_with_pose import AbstractWithPose
from arcor2_kinali.object_types.aubo import Aubo
from arcor2_kinali.object_types.barcode import Barcode
from arcor2_kinali.object_types.interaction import Interaction
from arcor2_kinali.object_types.search import Search
from arcor2_kinali.object_types.simatic import Simatic
from arcor2_kinali.object_types.statistic import Statistic


def main() -> None:

    # abstract classes
    upload_def(AbstractRobot)
    upload_def(AbstractSimple)
    upload_def(AbstractWithPose)

    # concrete classes
    upload_def(Barcode)
    upload_def(Interaction)
    upload_def(Aubo, urdf=Urdf(get_data("aubo"), Aubo.urdf_package_name))
    upload_def(Simatic)
    upload_def(Search)
    upload_def(Statistic)


if __name__ == "__main__":
    main()



from dataclasses import dataclass
from typing import List, Optional

from dataclasses_jsonschema import JsonSchemaMixin

from arcor2 import rest
from arcor2.data.common import ActionMetadata

from .abstract_simple import AbstractSimple


@dataclass
class StatisticValue(JsonSchemaMixin):

    value: float
    created: int


class Statistic(AbstractSimple):
    """Statistic Web API Reference (0.3.0)."""

    _ABSTRACT = False

    def get_names(self, group_id: str, *, an: Optional[str] = None) -> List[str]:
        """Gets names of all tracked values stored in given group.

        :param group_id:
        :return:
        """
        return rest.call(rest.Method.GET, f"{self.settings.url}/values/{group_id}", list_return_type=str)

    def add_value(self, group_id: str, name: str, value: float, *, an: Optional[str] = None) -> None:
        """Logs value with the specified group and name.

        :param group_id:
        :param name:
        :param value:
        :return:
        """

        rest.call(
            rest.Method.PUT, f"{self.settings.url}/values", params={"group_id": group_id, "name": name, "value": value}
        )

    def get_groups(self, *, an: Optional[str] = None) -> List[str]:
        """Gets Ids of all stored groups.

        :param name:
        :return:
        """

        return rest.call(rest.Method.GET, f"{self.settings.url}/values", list_return_type=str)

    def get_values(
        self, group_id: str, name: str, since_timestamp: int = 0, *, an: Optional[str] = None
    ) -> List[StatisticValue]:
        """Gets tracked values with the specified name. Values are sorted as
        were added to service.

        :param group_id: Logged value name.
        :param name: Logged value name.
        :param since_timestamp: The date and time, as a UNIX timestamp in nanoseconds, after which created values
                                are returned.
        :return:
        """

        return rest.call(
            rest.Method.GET,
            f"{self.settings.url}/values/{group_id}/{name}",
            list_return_type=StatisticValue,
            params={"since_timestamp": since_timestamp},
        )

    def delete_group(self, group_id: str, *, an: Optional[str] = None) -> None:
        """Deletes all tracked values stored in given group.

        :param group_id:
        :return:
        """

        rest.call(rest.Method.DELETE, f"{self.settings.url}/values/{group_id}")

    get_names.__action__ = ActionMetadata(blocking=True)  # type: ignore
    add_value.__action__ = ActionMetadata(blocking=True)  # type: ignore
    get_groups.__action__ = ActionMetadata(blocking=True)  # type: ignore
    get_values.__action__ = ActionMetadata(blocking=True)  # type: ignore
    delete_group.__action__ = ActionMetadata(blocking=True)  # type: ignore



from typing import Optional

from arcor2 import rest
from arcor2.data.common import ActionMetadata

from .abstract_with_pose import AbstractWithPose


class Barcode(AbstractWithPose):
    """REST interface to the barcode service (0.3.0)."""

    _ABSTRACT = False

    def scan(self, *, an: Optional[str] = None) -> str:
        """Gets scan from active scanner.

        :return:
        """

        return rest.call(rest.Method.GET, f"{self.settings.url}/scanner/scan", return_type=str)

    scan.__action__ = ActionMetadata(blocking=True)  # type: ignore



from typing import Set

from arcor2.data.common import Pose
from arcor2.exceptions import Arcor2NotImplemented
from arcor2.object_types.abstract import RobotType

from .abstract_robot import AbstractRobot


class Simatic(AbstractRobot):
    """REST interface to the robot service (0.7.0)."""

    _ABSTRACT = False
    robot_type = RobotType.CARTESIAN

    def get_end_effectors_ids(self) -> Set[str]:
        return set()

    def get_end_effector_pose(self, end_effector: str) -> Pose:
        raise Arcor2NotImplemented("Not supported")

    def suctions(self) -> Set[str]:
        return set()




# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: src/python/arcor2_kinali:arcor2_kinali_dist

from setuptools import setup

setup(**{
    'author': 'Robo@FIT',
    'author_email': 'imaterna@fit.vut.cz',
    'classifiers': [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Programming Language :: Python :: 3.8',
        'Natural Language :: English',
        'Topic :: Scientific/Engineering',
    ],
    'description': 'ARCOR2 integration for Kinali services.',
    'license': 'LGPL',
    'long_description': """# arcor2_kinali
# Changelog

The format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),

## [0.11.0] - 2021-02-08

### Changed
- Added the mandatory `an` parameter to all actions.

### Fixed
- Invalid url in `move_relative_joints`.

## [0.10.0] - 2020-12-14

### Changed
- Class `KinaliRobot` divided into `Aubo` and `Simatic`.
- Added example of the main script using `Resources` class.
- Added URDF model for `Aubo`.
- Updated to for Robot 0.9.0, Search 0.7.0, Barcode 0.4.0.

## [0.9.2] - 2020-10-30

### Changed
- Call to `patch_object_actions` added to the example script.
- Slight change of `inverse_kinematics` method signature (`start_joints` and `avoid_collisions` now have default values).

## [0.9.1] - 2020-10-19

### Changed
- ARCOR2 dependency updated

## [0.9.0] - 2020-10-16
### Changed
- Clients updated to use rewritten `rest` module.
- KinaliRobot updated for Robot 0.8.0 (IK/FK methods added).
- Barcode, Robot and Search now have pose while Interaction and Statistic do not.

### Fixed
- Interaction/add_dialog now returns the selected option.
- Barcode: obsolete method `active_scanners` removed.

## [0.8.0] - 2020-09-24
### Changed
- Package is now part of the arcor2 monorepo.
- Update of ObjectTypes.""",
    'long_description_content_type': 'text/markdown',
    'name': 'arcor2_kinali',
    'python_requires': '==3.8.*',
    'version': '0.11.0',
    'package_dir': {
        '': 'src',
    },
    'packages': (
        'arcor2_kinali',
        'arcor2_kinali.object_types',
        'arcor2_kinali.scripts',
    ),
    'namespace_packages': (
    ),
    'package_data': {
        'arcor2_kinali': (
            'VERSION',
            'data/aubo/aubo_i5.urdf',
            'data/aubo/aubo_i5/meshes/collision/base_link.STL',
            'data/aubo/aubo_i5/meshes/collision/foreArm_Link.STL',
            'data/aubo/aubo_i5/meshes/collision/pedestal.STL',
            'data/aubo/aubo_i5/meshes/collision/shoulder_Link.STL',
            'data/aubo/aubo_i5/meshes/collision/upperArm_Link.STL',
            'data/aubo/aubo_i5/meshes/collision/wrist1_Link.STL',
            'data/aubo/aubo_i5/meshes/collision/wrist2_Link.STL',
            'data/aubo/aubo_i5/meshes/collision/wrist3_Link.STL',
            'data/aubo/aubo_i5/meshes/visual/base_link.DAE',
            'data/aubo/aubo_i5/meshes/visual/foreArm_Link.DAE',
            'data/aubo/aubo_i5/meshes/visual/shoulder_Link.DAE',
            'data/aubo/aubo_i5/meshes/visual/upperArm_Link.DAE',
            'data/aubo/aubo_i5/meshes/visual/wrist1_Link.DAE',
            'data/aubo/aubo_i5/meshes/visual/wrist2_Link.DAE',
            'data/aubo/aubo_i5/meshes/visual/wrist3_Link.DAE',
            'py.typed',
        ),
    },
    'install_requires': (
        'Pillow==8.1.0',
        'arcor2~=0.11.0',
        'dataclasses-jsonschema-rereleased[fast-validation]==2.12.2',
    ),
    'entry_points': {
        'console_scripts': [
            'arcor2_kinali_upload_objects=arcor2_kinali.scripts.upload_objects:main',
        ],
    },
})



from dataclasses import dataclass
from typing import List, Optional

from dataclasses_jsonschema import JsonSchemaMixin
from PIL.Image import Image

from arcor2 import rest
from arcor2.data.common import ActionMetadata, Pose, StrEnum

from .abstract_with_pose import AbstractWithPose


@dataclass
class ScoredItem(JsonSchemaMixin):

    id: int
    item_pose: Pose
    search_score: float


@dataclass
class SearchOutput(JsonSchemaMixin):

    items: List[ScoredItem]
    container_pose: Pose


@dataclass
class GripperSetup(JsonSchemaMixin):

    pick_speed: float
    pre_pick_opening: float
    pick_opening: float
    pick_force: float
    pick_pose: Pose


@dataclass
class SearchVisualizationSetup(JsonSchemaMixin):

    container: bool
    space_selector: bool
    estimate: bool
    refine: bool


@dataclass
class SearchSaveInfo(JsonSchemaMixin):

    save_input: bool
    save_visualizations: bool
    path: Optional[str] = None


class LogLevel(StrEnum):

    TRACE: str = "trace"
    DEBUG: str = "debug"
    INFO: str = "info"
    WARN: str = "warn"
    ERROR: str = "error"
    FATAL: str = "fatal"


@dataclass
class PickArgs(JsonSchemaMixin):

    pick_pose: Pose


@dataclass
class SearchLogLevel(JsonSchemaMixin):

    level: LogLevel


@dataclass
class SearchEngineParameters(JsonSchemaMixin):

    visualization_setup: Optional[SearchVisualizationSetup] = None
    search_data_save_info: Optional[SearchSaveInfo] = None
    search_log_level: Optional[SearchLogLevel] = None


class Search(AbstractWithPose):
    """REST interface to the search service (0.5.0)."""

    _ABSTRACT = False

    # --- Capture Controller -------------------------------------------------------------------------------------------

    def grab_image(self, *, an: Optional[str] = None) -> None:
        """Grabs image and stores to internal cache.

        :return:
        """
        rest.call(rest.Method.PUT, f"{self.settings.url}/capture/grab")

    def get_image(self, *, an: Optional[str] = None) -> Image:
        """Gets RGB image from internal cache, if there is any.

        :return:
        """

        return rest.get_image(f"{self.settings.url}/capture/image")

    def get_pose(self, *, an: Optional[str] = None) -> Pose:
        """Gets capture device pose in actual initialized spatial system space.

        :return:
        """
        return rest.call(rest.Method.GET, f"{self.settings.url}/capture/pose", return_type=Pose)

    def capture_export(self, path_to_save_zip: str, *, an: Optional[str] = None) -> None:
        """Gets the zipped captured data.

        :param path_to_save_zip: Where to save the zip file.
        :return:
        """

        rest.download(f"{self.settings.url}/capture/export", path_to_save_zip)

    # --- Pick Controller ----------------------------------------------------------------------------------------------

    def upsert_pick_args(
        self, item_id: str, suction_id: str, poses: List[PickArgs], *, an: Optional[str] = None
    ) -> None:
        """Adds or updates arguments required for pick given item with given
        suction.

        :param item_id:
        :param suction_id:
        :param poses:
        :return:
        """
        rest.call(
            rest.Method.PUT,
            f"{self.settings.url}/pick/suctions",
            body=poses,
            params={"itemId": item_id, "suctionId": suction_id},
        )

    def compute_pick_args(
        self, item_id: str, suction_id: str, pose: Pose, *, an: Optional[str] = None
    ) -> List[PickArgs]:
        """Computes pick arguments for given suction, item and item pose.

        :param item_id:
        :param suction_id:
        :param pose:
        :return:
        """
        return rest.call(
            rest.Method.PUT,
            f"{self.settings.url}/pick/suctions/compute",
            body=pose,
            params={"itemId": item_id, "suctionId": suction_id},
            list_return_type=PickArgs,
        )

    def upsert_gripper_args(
        self, item_id: str, gripper_id: str, gripper_setup: List[GripperSetup], *, an: Optional[str] = None
    ) -> None:
        """Adds or updates arguments required for pick given item with given
        gripper.

        :param item_id:
        :param gripper_id:
        :param gripper_setup:
        :return:
        """

        rest.call(
            rest.Method.PUT,
            f"{self.settings.url}/pick/grippers",
            body=gripper_setup,
            params={"itemId": item_id, "gripperId": gripper_id},
        )

    def compute_gripper_args(
        self, item_id: str, gripper_id: str, pose: Pose, *, an: Optional[str] = None
    ) -> List[GripperSetup]:
        """Gets pick poses for specific tool and item.

        :param item_id:
        :param gripper_id:
        :param pose:
        :return:
        """

        return rest.call(
            rest.Method.GET,
            f"{self.settings.url}/pick/grippers/compute",
            list_return_type=GripperSetup,
            body=pose,
            params={"itemId": item_id, "gripperId": gripper_id},
        )

    # --- Search Controller --------------------------------------------------------------------------------------------

    def search(self, *, an: Optional[str] = None) -> SearchOutput:
        """Searches items based on search engine configuration and images
        stored in internal cache.

        :return:
        """
        return rest.call(rest.Method.GET, f"{self.settings.url}/search", return_type=SearchOutput)

    def set_search_parameters(self, parameters: SearchEngineParameters, *, an: Optional[str] = None) -> None:
        """Sets the search parameters.

        :param parameters:
        :return:
        """

        rest.call(rest.Method.PUT, f"{self.settings.url}/search", body=parameters)

    def visualization(self, *, an: Optional[str] = None) -> Image:
        """Gets RGB visualization from last search run, if there is any.

        :return:
        """

        return rest.get_image(f"{self.settings.url}/search/visualization")

    grab_image.__action__ = ActionMetadata(blocking=True)  # type: ignore
    get_image.__action__ = ActionMetadata(blocking=True)  # type: ignore
    get_pose.__action__ = ActionMetadata(blocking=True)  # type: ignore

    upsert_pick_args.__action__ = ActionMetadata(blocking=True)  # type: ignore
    compute_pick_args.__action__ = ActionMetadata(blocking=True)  # type: ignore
    upsert_gripper_args.__action__ = ActionMetadata(blocking=True)  # type: ignore
    compute_gripper_args

    search.__action__ = ActionMetadata(blocking=True)  # type: ignore
    set_search_parameters.__action__ = ActionMetadata(blocking=True)  # type: ignore
    visualization.__action__ = ActionMetadata(blocking=True)  # type: ignore
